#!/usr/bin/python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# Copyright (c) 2012 Michael Hull.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------

import sys
import os
import cStringIO
import time
import traceback
import datetime
import signal
import shutil
import pipes

# Setup django:
from django.core.management import setup_environ
import mreorg.curator.settings as settings
setup_environ(settings)

import mreorg
from mreorg.utils import get_file_sha1hash
from mreorg.curator.frontend.models import SimFileRun
from mreorg.curator.frontend.models import SimFile
from mreorg.curator.frontend.models import SimFileRunOutputImage
from mreorg.curator.frontend.models import RunConfiguration
from mreorg.curator.frontend.models import TrackingStatus
from django.db import transaction


class SimDBWriter(object):

    @classmethod
    def write_to_database(cls, sim_run_info):
        output_file_dir = mreorg.MReOrgConfig.get_image_store_dir()

        print 'Saving details from script: ', sim_run_info.script_name

        # We don't neeed to update this file every time:
        if mreorg.MReOrgConfig.is_non_curated_file(sim_run_info.script_name):
            return


        simfile = SimFile.get_or_make(
							full_filename=sim_run_info.script_name,
							make_kwargs={'tracking_status': TrackingStatus.Tracked})
        

        # Create a simulation result object:
        run_id = int(os.environ['_MREORG_RUNCONFIGID'])
        simres = SimFileRun(
            simfile=simfile,
            execution_date=datetime.datetime.now(),
            execution_time=sim_run_info.time_taken,
            return_code=sim_run_info.return_code,
            std_out=sim_run_info.std_out,
            std_err=sim_run_info.std_err,
            exception_type=sim_run_info.exception_details[0],
            exception_traceback=str(sim_run_info.exception_details[2]),
            simulation_sha1hash=get_file_sha1hash(simfile.full_filename),
            library_sha1hash='00000',

            runconfig=RunConfiguration.objects.get(id=run_id),
            )

        simres.save()

        with transaction.commit_on_success():
            output_file_dir = mreorg.MReOrgConfig.get_image_store_dir()
            # Create the images
            for image_filename in sim_run_info.output_images:
                if not image_filename.endswith('svg'):
                    continue

                # Copy the output file:
                try:
                    hashstr = mreorg.get_file_sha1hash(image_filename)
                except:
                    hashstr = None

                if hashstr == None:
                    continue

                opfile1 = output_file_dir + '/' + hashstr + '.svg'
                shutil.copyfile(image_filename, opfile1)

                f_thumb = image_filename.replace('.svg', 'thumb.png')
                os.system('convert %s -resize 400x300 %s' % (pipes.quote(image_filename),pipes.quote(f_thumb)))
                time.sleep(5) # Sometimes, its not ready!
                hashstr = mreorg.get_file_sha1hash(f_thumb)
                opfile2 = output_file_dir + '/' + hashstr + '.png'
                shutil.copyfile(f_thumb, opfile2)
                im_obj = SimFileRunOutputImage(
                        original_name=image_filename,
                        hash_name=opfile1,
                        hash_thumbnailname=opfile2,
                        simulation=simres)

                im_obj.save()


class SimRunInfo(object):

    def __init__(self, script_name):
        self.return_code = None
        self.time_taken = None
        self.time_out = None
        self.std_out = None
        self.std_err = None
        self.exception_details = (None, None)

        self.script_name = script_name
        self.output_images = []


class IOStreamDistributor(object):

    def __init__(self, outputs):
        self.outputs = outputs

    def write(self, *args, **kwargs):
        for output in self.outputs:
            output.write(*args, **kwargs)

    def flush(self):
        for output in self.outputs:
            try:
                output.flush()
            except:
                pass


class TimeoutException(Exception):

    def __init__(self, timeout):
        super(TimeoutException, self).__init__()
        self.timeout = timeout

    def __repr__(self):
        return 'TImeoutException (%ds)' % self.timeout

    def __str__(self):
        return 'TImeoutException (%ds)' % self.timeout


class CurationSimDecorator(object):

    start_time = None
    time_out = None
    is_initialised = False
    exception_details = (None, None, None)

    std_out = None
    std_err = None
    script_name = None

    @classmethod
    def exit_handler(cls, *_args, **_kwargs):

        info = SimRunInfo(cls.script_name)

        # Read and restore the StdOut/Err
        info.std_out = cls.std_out.getvalue()
        sys.stdout = sys.__stdout__
        info.std_err = cls.std_err.getvalue()
        sys.stderr = sys.__stderr__

        # Pick-up any saved images:
        mreorg.PlotManager.save_active_figures()
        info.output_images = mreorg.PlotManager.figures_saved_filenames

        # Get the return value:
        info.return_code = 0

        # Get the timing:
        info.time_taken = int(time.time() - cls.start_time)

        # Has there been an exception?
        info.exception_details = cls.exception_details
        if info.exception_details != (None, None, None):
            print 'Exception details', info.exception_details
            info.return_code = -1

        # Write to SimDataBase
        SimDBWriter.write_to_database(info)

    @classmethod
    def top_level_exception_handler(
        cls,
        exception_type,
        exception,
        tracebackobj,
        *_args
        ):
        try:
            traceback_str = str(exception) + '\n' \
                + ''.join(traceback.format_tb(tracebackobj))

            print ''
            print 'TopLevel-Handler Caught Exception:'
            print '----------------------------------'
            print traceback_str
            cls.exception_details = (exception_type, 
                                     exception,
                                     traceback_str)
            cls.exit_handler()
        except Exception, exception:
            print 'INTERNAL ERROR, exception raised in top level handler!'
            print exception
            sys.exit(0)

    @classmethod
    def activate(cls, time_out=None):
        assert not cls.is_initialised

        if 'MREORG_TIMEOUT' in os.environ:
            time_out = int(os.environ['MREORG_TIMEOUT'])

        # Filename of the Sim script
        cwd = os.getcwd()
        cls.script_name = os.path.join(cwd, traceback.extract_stack()[0][0])

        # Intercept StdOut and StdErr:
        cls.std_out = cStringIO.StringIO()
        sys.stdout = IOStreamDistributor([cls.std_out, sys.stdout])
        cls.std_err = cStringIO.StringIO()
        sys.stderr = IOStreamDistributor([cls.std_err, sys.stderr])

        # Set an exit handler
        from mhutils.atexithandlers import AtExitHandler
        AtExitHandler.enable()
        AtExitHandler.add_handler(cls.exit_handler, 100)

        # Set a top level exception handler:
        sys.excepthook = cls.top_level_exception_handler

        # Set a time-out alarm
        cls.start_time = time.time()
        cls.time_out = time_out

        def timeout_sighandler(_signum, _frame):
            raise TimeoutException(timeout=cls.time_out)

        if time_out:
            signal.signal(signal.SIGALRM, timeout_sighandler)
            signal.alarm(time_out)





#!/usr/bin/python
# -*- coding: utf-8 -*-

import argparse
import sys
import os
from django.core.management import execute_from_command_line

if not 'MREORG_CONFIG' in os.environ:
    os.environ['MREORG_CONFIG'] = ''

# django internally will call this file, when files on the disk change.
# To allow this; we set a flag as an environmental variable, and
# check to see if we are 're-entering' when running the webserver:
if 'CURATION_REENTRYFLAG' in os.environ['MREORG_CONFIG']:
    print 'Rentry'
    execute_from_command_line(sys.argv)


def ensure_MREORG_REENTRY_flag_set():
    # Make sure that the 'CURATION_REENTRYFLAG' is in the list of flags:
    mreorg_config_str = os.environ.get('MREORG_CONFIG', '')
    if not 'CURATION_REENTRYFLAG' in mreorg_config_str:
        mreorg_config_str += ';CURATION_REENTRYFLAG'
    os.environ['MREORG_CONFIG'] = mreorg_config_str


def cmd_runserver(params):
    os.environ['DJANGO_SETTINGS_MODULE'] = 'mreorg.curator.settings'

    # Make sure that the 'CURATION_REENTRYFLAG' is in the list of flags:
    ensure_MREORG_REENTRY_flag_set()

    # OK....
    if params.profile:
        print 'Building args'
        sys.argv = [__file__, 'runprofileserver', '%d' % params.port, ' --kcachegrind', ' --prof-path=/tmp/my-profile-data' ]
    else:
        sys.argv = [__file__, 'runserver', '%d' % params.port]
    print sys.argv
    execute_from_command_line(sys.argv)


def cmd_runbackend(params):
    import mreorg.curator.backend.simmgr_backend
    mreorg.curator.backend.simmgr_backend.run_backend()


def cmd_builddb(params):
    print 'Rebuild-db'
    import mreorg
    db_filename = mreorg.MReOrgConfig.get_simulation_sqllite_filename()

    # Make sure that the 'CURATION_REENTRYFLAG' is in the list of flags:
    ensure_MREORG_REENTRY_flag_set()
    if params.rebuild:
        os.unlink(db_filename)
    if os.path.exists(db_filename):
        raise ValueError('The database already exists. Either delete it manually or use the --rebuild flag to delete the existing data')

    os.environ['DJANGO_SETTINGS_MODULE'] = 'mreorg.curator.settings'
    sys.argv = [__file__, 'syncdb']
    execute_from_command_line(sys.argv)


def cmd_backup(*args, **kwargs):
    print 'backup', args, kwargs
    assert False, 'Not Implemented yet'


def cmd_reloadconfig(params):
    os.environ['DJANGO_SETTINGS_MODULE'] = 'mreorg.curator.settings'

    from mreorg.curator.frontend.dbdata_from_config import update_db_from_config
    update_db_from_config()


def build_parser():
    description = 'mreorg.curate is tool for managing large numbers of simulations'
    parser = argparse.ArgumentParser(description=description)
    subparsers = parser.add_subparsers(help='sub-command help')

    parser_reloadconfig = subparsers.add_parser('reconfig', help='Reload the configuration from the configuration file' )
    parser_reloadconfig.set_defaults(func=cmd_reloadconfig)

    parser_runserver = subparsers.add_parser('runserver', help='run the django backend' )
    parser_runserver.add_argument('-p', '--port', type=int, default=8000, help='the port to run django on')
    parser_runserver.add_argument('--profile', action='store_true', help='profile the server using django-extensions')
    parser_runserver.set_defaults(func=cmd_runserver)


    parser_runbackend = subparsers.add_parser('runbackend', help="launch a 'backend-worker")
    parser_runbackend.set_defaults(func=cmd_runbackend)

    parser_builddb = subparsers.add_parser('builddb', help='create the database')
    parser_builddb.add_argument('-r', '--rebuild',action='store_true',  help='delete an existing database before starting')
    parser_builddb.set_defaults(func=cmd_builddb)

    parser_backup = subparsers.add_parser('backup', help='backup the existing database')
    parser_backup.set_defaults(func=cmd_backup)
    return parser


def main():
    parser = build_parser()
    args = parser.parse_args()
    args.func(args)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

import models
from django.db import transaction
import os
import mreorg
import glob2


@transaction.commit_on_success
def update_all_db(directory):

    print 'Updating untracked simulation files', directory
    for (dirpath, dirnames, filenames) in os.walk(directory):
        for filename in sorted(filenames):
            if not filename.endswith('.py'):
                continue
            print filename
            full_filename = os.path.join(dirpath, filename)
            if mreorg.MReOrgConfig.is_non_curated_file(filename):
                continue
            if mreorg.MReOrgConfig.is_non_curated_file(full_filename):
                continue

            print ' -- Adding:', filename
            models.SimFile.get_or_make(full_filename=full_filename, make_kwargs={'tracking_status':models.TrackingStatus.NotTracked})
            print 'Added OK!'


def update_db_from_config():
    import mreorg
    from mreorg.curator.frontend.models import RunConfiguration, FileGroup, SimFile
    from mreorg.curator.frontend.models import EnvironVar

    # Update the FileGroups:
    with transaction.commit_on_success():
        try:
            default_filegroups = mreorg.MReOrgConfig.config['CurateAutoloadData']['FileGroups']
        except KeyError:
            default_filegroups = {}

        for fgname, fgglobs in default_filegroups.iteritems():
            filenames = set()
            for fgglob in fgglobs:
                filenames.update(glob2.glob(fgglob))

            # Safely get the FileGroup:
            fg = FileGroup.get_or_make(name=fgname)
            assert not fg.is_special(), 'Trying to overwrite a builtin filegroup'
            for filename in filenames:
                simfile = SimFile.get_or_make(full_filename=filename)
                if not fg.contains_simfile(simfile):
                    fg.simfiles.add(simfile)
                    fg.save()

            print 'Updated FileGroup: %s' % (fgname, )

    # Update the RunConfigurations:
    with transaction.commit_on_success():
        try:
            default_runconfigs = mreorg.MReOrgConfig.config['CurateAutoloadData']['RunConfigs']
        except KeyError:
            default_runconfigs = {}
        for confname, confinfo in default_runconfigs.iteritems():
            runconf = RunConfiguration.get_or_make(name=confname)
            assert not runconf.is_special(), 'Trying to add a builtin-configuration'
            runconf.timeout = confinfo.get('timeout',None)

        for (key, value) in confinfo.get('env_vars',{}).iteritems():
            try:
                envvar = runconf.environvar_set.get(key=key)
                envvar.value = value
                envvar.save()
            except EnvironVar.DoesNotExist:
                envvar = EnvironVar(key=key,value=value,config=runconf)
                envvar.save()

        runconf.save()
        print 'Updated RunConfig: %s' % confname

        # Add default locations:
        mh_adddefault_locations()
        # Rescan-filesystem:
        rescan_filesystem()

        # Cache docstring
        print 'Pre-Caching Docstrings:'
        for simfile in SimFile.objects.all():
            print simfile.full_filename
            simfile.get_docstring()

        print 'Finished Reconfiguring'


def rescan_filesystem():
    import dbdata_from_config

    with transaction.commit_on_success():
        for src_dir in models.SourceSimDir.objects.all():
            dbdata_from_config.update_all_db(src_dir.directory_name)


def mh_adddefault_locations():
    import mreorg
    default_simulations = mreorg.MReOrgConfig.config['Settings']['Curate']['default_tracked_simulations']

    with transaction.commit_on_success():
        for l in default_simulations:
            if not l:
                continue
            if l[-1] != '*':
                models.SimFile.get_or_make(full_filename=l,
                        make_kwargs={'tracking_status': models.TrackingStatus.Tracked})
            elif l.endswith('**'):
                models.SourceSimDir.create(directory_name=l[:-2],
                        should_recurse=True)
            elif l.endswith('*'):
                models.SourceSimDir.create(directory_name=l[:-1],
                        should_recurse=False)
            else:
                models.SourceSimDir.create(directory_name=l[:-2],
                        should_recurse=False)





#!/usr/bin/python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# Copyright (cxt_data) 2012 Michael Hull.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------

from django.shortcuts import render_to_response

from django.http import HttpResponseRedirect, HttpResponseNotFound
from django.http import HttpResponse
from mreorg.curator.frontend.models import SimFile
from mreorg.curator.frontend.models import SimRunStatus
from mreorg.curator.frontend.models import SimFileRun
from mreorg.curator.frontend.models import SimQueueEntry
from mreorg.curator.frontend.models import SimQueueEntryState
from mreorg.curator.frontend.models import SourceSimDir
from mreorg.curator.frontend.models import RunConfiguration
from mreorg.curator.frontend.models import FileGroup
from mreorg.curator.frontend.models import TrackingStatus
from mreorg import MReOrgConfig
from django.template import RequestContext
from dbdata_from_config import rescan_filesystem

import os
import re
import string
from urlparse import urlsplit

import os.path
import mimetypes
mimetypes.init()

from django.db import transaction


def ensure_config(request):
    if not 'current_runconfig' in request.session:
        request.session['current_runconfig'] = RunConfiguration.get_initial()
    if not 'current_filegroup' in request.session:
        request.session['current_filegroup'] = FileGroup.get_initial()


def config_processor(request):
    ensure_config(request)

    return {
        'runconfigs': RunConfiguration.objects.all(),
        'filegroups': FileGroup.objects.all(),
        'current_runconfig': request.session['current_runconfig'],
        'current_filegroup': request.session['current_filegroup'],
        }


class SimFileWithRunConfigProxy(object):

    def __init__(self, simfile, runconfig):
        self.simfile = simfile
        self.runconfig = runconfig

        self._does_file_exist = self.simfile.does_file_exist()
        self._get_docstring= self.simfile.get_docstring()
        self._get_short_filename= self.simfile.get_short_filename()
        self._get_status= self.simfile.get_status(runconfig=self.runconfig)
        self._get_last_executiontime= self.simfile.get_last_executiontime(runconfig=self.runconfig)
        self._is_queued= self.simfile.is_queued(runconfig=self.runconfig)
        self._getCSSQueueState= self.simfile.getCSSQueueState(runconfig=self.runconfig)
        self._full_filename= self.simfile.full_filename
        self._id= self.simfile.id
        self._tracking_status= self.simfile.tracking_status
        self._get_last_run= self.simfile.get_last_run(runconfig=self.runconfig)
        self._is_currently_running= self.simfile.is_currently_running(self.runconfig)


    def does_file_exist(self):
        return self._does_file_exist

    def get_docstring(self):
        return self._get_docstring

    def get_short_filename(self):
        return self._get_short_filename

    def get_status(self):
        return self._get_status

    def get_last_executiontime(self):
        return self._get_last_executiontime

    def is_queued(self):
        return self._is_queued

    def getCSSQueueState(self):
        return self._getCSSQueueState

    @property
    def full_filename(self):
        return self._full_filename

    @property
    def id(self):
        return self._id

    @property
    def tracking_status(self):
        return self._tracking_status

    def get_last_run(self):
        return self._get_last_run

    def is_currently_running(self):
        return self._is_currently_running

    def __getattr__(self, name):
        print 'looking for attribute:', name
        assert False


def build_proxy_for_sim_files(simfiles, runconfig):
    return [SimFileWithRunConfigProxy(sim, runconfig) for sim in
            simfiles if sim.does_file_exist()]


def view_overview(request):
    ensure_config(request)

    sims = build_proxy_for_sim_files( request.session['current_filegroup'].tracked_files, runconfig= request.session['current_runconfig'] )
    return render_to_response(
            'overview.html',
            RequestContext(request,
                { 'simfiles': sims },
                [config_processor]
                )
            )

def view_sim_output_summaries(request):
    ensure_config(request)
    sims = build_proxy_for_sim_files( request.session['current_filegroup'].tracked_files, runconfig= request.session['current_runconfig'] )
    cxt_data = {'simfiles':sims}
    csrf_context = RequestContext(request, cxt_data, [config_processor] )
    return render_to_response('simulation_output_summaries.html', csrf_context)


def view_configurations(request):
    ensure_config(request)

    cxt_data = {}
    csrf_context = RequestContext(request, cxt_data, [config_processor])
    return render_to_response('configurations.html', csrf_context)


def simfilerun_details(request, run_id):
    return render_to_response(
            'simulation_run_details.html',
            RequestContext(request,
                {'simulationrun': SimFileRun.objects.get(id=run_id) },
                [config_processor] ) )



def simfile_details(request, simfile_id):
    sim_file = SimFile.get_tracked_sims(id=simfile_id)
    cxt_data = {'simfile': sim_file}
    csrf_context = RequestContext(request, cxt_data)
    return render_to_response('simfile_details.html', csrf_context)


def view_tracking(request):

    cxt_data = {'src_directories': SourceSimDir.objects.all(),
                'untracked_simfiles': SimFile.get_untracked_sims(),
                'simfiles': SimFile.get_tracked_sims()}
    csrf_context = RequestContext(request, cxt_data, [config_processor])
    return render_to_response('tracking.html', csrf_context)


# Tracking Commands
# ====================

@transaction.commit_on_success
def do_track_all(request):
    for sim in SimFile.get_untracked_sims():
        sim.tracking_status = TrackingStatus.Tracked
        sim.save()
    return HttpResponseRedirect('/tracking')


@transaction.commit_on_success
def do_untrack_all(request):
    for sim in SimFile.get_tracked_sims():
        sim.tracking_status = TrackingStatus.NotTracked
        sim.save()

    return HttpResponseRedirect('/tracking')


def do_track_src_dir(request):
    if request.method != 'POST':
        return HttpResponseRedirect('/tracking')

    SourceSimDir.create(directory_name=request.POST['location'],
			should_recurse='recurse' in request.POST)
    return HttpResponseRedirect('/tracking')


def do_untrack_src_dir(request, srcdir_id):
    o = SourceSimDir.objects.get(id=srcdir_id)
    o.delete()
    return HttpResponseRedirect('/tracking')


def do_track_rescanfs(request):
    rescan_filesystem()
    return HttpResponseRedirect('/tracking')


@transaction.commit_on_success
def do_track_sim(request):
    if not request.method == 'POST':
        return HttpResponseRedirect('/tracking')

    # Find all keys matching untracked_sim_id_XX, and get the XX's

    r = re.compile(r"""untracked_sim_id_(?P<id>\d+)""", re.VERBOSE)
    pot_sim_id_matches = [r.match(k) for k in request.POST]
    pot_sim_ids = [int(m.groupdict()['id']) for m in pot_sim_id_matches if m]

    for pot_sim_id in pot_sim_ids:
        sim = SimFile.objects.get(id=pot_sim_id)
        sim.tracking_status = TrackingStatus.Tracked
        sim.save()

    return HttpResponseRedirect('/tracking')


@transaction.commit_on_success
def do_untrack_sim(request):
    if not request.method == 'POST':
        return HttpResponseRedirect('/tracking')

    # Find all keys matching untracked_sim_id_XX, and get the XX's
    print request.POST.keys()
    r = re.compile(r"""simid_(?P<id>\d+)""", re.VERBOSE)
    pot_sim_id_matches = [r.match(k) for k in request.POST]
    pot_sim_ids = [int(m.groupdict()['id']) for m in pot_sim_id_matches
                   if m]

    # Delete the old simulations:
    for pot_sim_id in pot_sim_ids:
        sim = SimFile.objects.get(id=pot_sim_id)
        sim.tracking_status = TrackingStatus.NotTracked
        sim.save()

    # Update the list of untracked files:
    return do_track_rescanfs(request)


# ====================

def viewsimulationqueue(request):
    cxt_data = \
        {'simulation_queue_executing': SimQueueEntry.objects.filter(status=SimQueueEntryState.Executing),
         'simulation_queue': SimQueueEntry.objects.filter(status=SimQueueEntryState.Waiting),
         'latest_runs': SimFileRun.objects.order_by('-execution_date')[0:10]}

    SimQueueEntry.trim_dangling_jobs()

    csrf_context = RequestContext(request, cxt_data)
    return render_to_response('view_simulation_queue.html', csrf_context)


def view_simulation_failures(request):
    ensure_config(request)
    runconfig= request.session['current_runconfig']
    simfiles = build_proxy_for_sim_files(SimFile.get_tracked_sims(), runconfig=runconfig)
    cxt_data = {
        'failed_simulations': [fo for fo in simfiles if fo.get_status() == SimRunStatus.UnhandledException],
        'timeout_simulations': [fo for fo in simfiles if fo.get_status() == SimRunStatus.TimeOut],
        'nonzero_exitcode_simulations': [fo for fo in simfiles if fo.get_status() == SimRunStatus.NonZeroExitCode],
        'changed_simulations': [fo for fo in simfiles if fo.get_status() == SimRunStatus.FileChanged],
        'notrun_simulations': [fo for fo in simfiles if fo.get_status() == SimRunStatus.NeverBeenRun],
        }

    csrf_context = RequestContext(request, cxt_data, [config_processor])
    return render_to_response('view_simulation_failures.html',
                              csrf_context)


def doremovesimulationsfromqueue(request):
    return HttpResponseRedirect('/viewsimulationqueue')


def do_queue_add_sims(request):
    ensure_config(request)
    if not request.method == 'POST':
        return HttpResponseRedirect('/viewsimulationqueue')

    print 'Queuing Sims:'
    print request.POST
    r = re.compile(r"""simid_(?P<id>\d+)""", re.VERBOSE)
    sim_id_matches = [r.match(k) for k in request.POST]
    sim_ids = [int(m.groupdict()['id']) for m in sim_id_matches if m]

    runconfig = request.session['current_runconfig']

    for simfile_id in sim_ids:
        sim_file = SimFile.get_tracked_sims(id=simfile_id)

        qe = SimQueueEntry(simfile=sim_file, runconfig=runconfig)
        qe.save()

        # Avoid Duplication
    return HttpResponseRedirect('/viewsimulationqueue')


def doeditsimfile(request, simfile_id):

    # Open up the file in an editor:
    sim_file = SimFile.get_tracked_sims(id=simfile_id)

    cwd = os.getcwd()
    os.chdir(os.path.split(sim_file.full_filename)[0])
    data_dict = {'full_filename': sim_file.full_filename}
    cmds = MReOrgConfig.config['Settings']['Curate']['drop_into_editor_cmds']
    for cxt_data in cmds:
        t = string.Template(cxt_data).substitute(**data_dict)
        os.system(t)
    os.chdir(cwd)

    # Return to the previous page:
    referer = request.META.get('HTTP_REFERER', None)
    if referer is None:
        return HttpResponseRedirect('/')

    try:
        redirect_to = urlsplit(referer, 'http', False)[2]
        return HttpResponseRedirect(redirect_to)
    except IndexError:
        return HttpResponseRedirect('/')


def get_image_file(request, filename):

    im_dir = MReOrgConfig.get_image_store_dir()

    expected_filename = os.path.join(im_dir, filename)

    if os.path.exists(expected_filename):

        fsock = open(expected_filename, 'r')
        file_path = expected_filename
        file_name = os.path.basename(file_path)
        mime_type_guess = mimetypes.guess_type(file_name)
        if mime_type_guess is not None:
            response = HttpResponse(fsock, mimetype=mime_type_guess[0])
        response['Content-Disposition'] = 'attachment; filename=' \
            + file_name
        return response
    else:
        return HttpResponseNotFound('<h1>Image not found</h1>')
    assert False






import importlib

from mhutils.fs.packages import PythonPackage

def test_can_import_all():
    print "Running test1"


    pkg = PythonPackage('/home/michael/dev/mreorg/src/mreorg')
    for modulename in pkg.module_names:

        try:
            i = importlib.import_module(modulename)
            print "Successfully imported:", modulename
        except:
            print "Error importing:", modulename



if __name__=='__main__':
    test_can_import_all()



import os
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "mreorg",
    version = "0.2.0",
    author = "Mike Hull",
    author_email = "mikehulluk@gmail.com",
    description = ("Tools for organising and simplifying scripts for modelling"),
    license = "BSD",
    url = "https://github.com/mikehulluk/mreorg",

    package_dir = {'':'src' },
    packages=['mreorg',
              'mreorg.curator',
              'mreorg.curator.cmdline',
              'mreorg.curator.backend',
              'mreorg.curator.frontend',
              'mreorg.curator.frontend.templates',
              'mreorg.requiredpreimport',
              'mreorg.scriptplots',
              ],
    # Could also have been done with 'scripts=':
    entry_points = {
        'console_scripts': [
            'mreorg.curate = mreorg.curator.cmdline.mreorg_curate:main [curator,mpl]',
        ],
    },

    package_data={
        'mreorg':[
            'curator/frontend/templates/*.html',
            'curator/frontend/static/javascript/*',
            'curator/frontend/static/customcss/*.css',
            'curator/frontend/static/customcss/images/*',
            'curator/frontend/static/javascript-moment/*',
            'curator/frontend/static/sphinx/*',
            ]
        },


    data_files=[('mreorg/etc', ['etc/configspec.ini']),
                #('config', ['cfg/data.cfg']),
                #('/etc/init.d', ['init-script'])
                ],
    
    install_requires=['matplotlib','configobj','Django','pygments', 'django-dajaxice', 'mhutils'],

    extras_require = {
        'mpl':  ["matplotlib"],
        'curator': ["Django","django-dajaxice","pygments"],
    },



    long_description=read('README.txt'),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License",
    ],
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# Copyright (c) 2012 Michael Hull.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------

import os
import mreorg.config
this_dir = os.path.dirname(__file__)
MREORG_ROOTDIR = os.path.join(this_dir, '../../../')

# Django settings for simmgr project.

DEBUG = True
TEMPLATE_DEBUG = DEBUG

ADMINS = ()

MANAGERS = ADMINS

db_filename = mreorg.config.MReOrgConfig.get_simulation_sqllite_filename()
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': db_filename,
        'OPTIONS': {
            'timeout': 120,
        }
    }
}



TIME_ZONE = None
LANGUAGE_CODE = 'en-uk'

SITE_ID = 1

USE_I18N = True
USE_L10N = True

# Make this unique, and don't share it with anybody.
SECRET_KEY = '$(!_gcohvlnbhbo_m!+(ew0(ggf0!c%$1&bv9a!5vs3%h2=ytt'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)
SESSION_SERIALIZER = 'django.contrib.sessions.serializers.PickleSerializer'

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
)

ROOT_URLCONF = 'mreorg.curator.urls'

TEMPLATE_DIRS = (
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    MREORG_ROOTDIR + 'src/mreorg/curator/frontend/templates/',
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
    'django.template.loaders.eggs.Loader'
)

INSTALLED_APPS = (
    'django.contrib.staticfiles',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'mreorg.curator.frontend',
    'django.contrib.humanize',
    'dajaxice',
    'django_extensions',
    )

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.core.context_processors.static',
    'django.core.context_processors.request',
    'django.contrib.messages.context_processors.messages',
    )

DAJAXICE_MEDIA_PREFIX = 'dajaxice'

STATICFILES_FINDERS = \
    ('django.contrib.staticfiles.finders.FileSystemFinder',
     'django.contrib.staticfiles.finders.AppDirectoriesFinder',
     'dajaxice.finders.DajaxiceFinder')

import logging
logging.basicConfig(level=logging.WARNING)

STATIC_URL = '/static/'

